(function(window){
	'use strict'

	function ranger()
	{
		var library = {};

		var settings = {
			class : null,
			parentClass :  null,
			range : [],
			unit : [],
			height : 0,
			labelsWidth : 60,
			thumbWidth : 30,
			thumbHeight : 30,
			trackHeight : 20,
			spacing : 'equidistant',
			skipFirst : false,
			everyOtherLabel : false,
			startPosition: {},
			endPosition: Infinity,
			markerPosition: {},
			isMouseDown : {},
			results: {},
		};

		library.init = function(options){

			console.info('initialized - Ranger');

			if( options.hasOwnProperty('class') ){

				if( typeof options.class === 'string' ) {
				    settings.class = options.class;
				} else {
					console.error('Uncaught ReferenceError: ' + typeof(options.class));
				}

			} else {
				console.error('Uncaught ReferenceError: class required');
			}

			if( options.hasOwnProperty('parentClass') ){

				if( typeof options.parentClass === 'string' ) {
				    settings.parentClass = options.parentClass;
				} else {
					console.error('Uncaught ReferenceError: ' + typeof(options.class));
				}	

			} else {
				console.error('Uncaught ReferenceError: parent class required');
			}

			if( options.hasOwnProperty('range') ){

				if( typeof options.range === 'object' ) {
				    settings.range = options.range;
				} else {
					console.error('Uncaught ReferenceError: ' + typeof(options.range));
				}	

			} else {
				console.error('Uncaught ReferenceError: range required');
			}

			if( options.hasOwnProperty('unit') ){

				if( typeof options.unit === 'object' ) {
				    settings.unit = options.unit;
				} else {
					console.error('Uncaught ReferenceError: ' + typeof(options.unit));
				}	
			} else {
				console.error('Uncaught ReferenceError: unit required');
			}

			if( typeof settings.class === 'string' ){
				getElements();
			}

			localStorage.setItem('rangerResults', JSON.stringify(settings.selected));
		}

		function getElements()
		{	
			var parents = document.querySelectorAll('.' + settings.parentClass);

			if(settings.range.length != settings.unit.length){
				console.error('Uncaught ReferenceError: range/unit/sliders/parents have to be equal');
			} else {

				// slider settings
				for (var i = 0; i < parents.length; i++){

					(function(pindex){

						var ranges = settings.range[pindex].sort(function(a, b){return a - b});
						var units = settings.unit[pindex];
						var key = "slider_" + pindex;

						var totalWidth = parents[pindex].offsetWidth;

						// Label Positioning
						var thumbOffset = settings.thumbWidth / (settings.everyOtherLabel ?  1 : 2) - settings.thumbWidth / 2;
						var labelOffset = settings.labelsWidth / (settings.everyOtherLabel ?  1 : 2) - settings.thumbWidth / 2;
						var labelDistanceOffset = settings.thumbWidth;
						var total = ranges.length - 1;
						var half = total / 2;

						var labelSmallestDistance = getSmallestDistance(labelDistanceOffset, ranges);
						var sliderSmallestDistance = getSmallestDistance(labelDistanceOffset, ranges);

						var labelContainer =  document.createElement('div');
						labelContainer.className = 'pointer';
						var labelWrapper = document.createElement('ul');

						settings.markerPosition[key] = [];
						settings.isMouseDown[key] = false;

						for (var j = settings.skipFirst ? 1 : 0; j <= total; settings.everyOtherLabel ? j += 2 : j++) {

							settings.markerPosition[key].push( ( totalWidth / total) * j );

							var slider = document.createElement('span');
							slider.className = 'slider';
							slider.style.marginTop = - settings.thumbHeight / 2 + "px";
							slider.style.left = "calc(" + sliderSmallestDistance + " * " + j + " - " + thumbOffset + "px)";
							slider.style.left = "-webkit-calc(" + sliderSmallestDistance + " * " + j + " - " + thumbOffset + "px)";
							slider.style.left = "-moz-calc(" + sliderSmallestDistance + " * " + j + " - " + thumbOffset + "px)";

							if(j == 0){
								slider.classList.add('tactive');
							}

							parents[pindex].appendChild(slider);

							var labelElement = document.createElement('li');
							labelElement.style.width = settings.labelsWidth * (settings.everyOtherLabel ? 2 : 1) + "px";
							labelElement.id = settings.class + '_label_' + j;
							labelElement.dataset.index = j;

							labelElement.dataset.slider = pindex;
							labelElement.style.left = "calc(" + labelSmallestDistance + " * " + j + " - " + labelOffset + "px)";
							labelElement.style.left = "-webkit-calc(" + labelSmallestDistance + " * " + j + " - " + labelOffset + "px)";
							labelElement.style.left = "-moz-calc(" + labelSmallestDistance + " * " + j + " - " + labelOffset + "px)";

							labelElement.onclick = function(){
								hideThumbPosition(parents[pindex], this.dataset.index);
								settings.results[key] = ranges[this.dataset.index];
								localStorage.setItem('rangerResults', JSON.stringify(settings.results));
							}

							var span = document.createElement('span');
							span.innerText = units[j];

							labelElement.appendChild(span);
							labelWrapper.appendChild(labelElement);
						}

						labelContainer.appendChild(labelWrapper);
						parents[pindex].parentElement.appendChild(labelContainer);

						parents[pindex].onmousedown = function(e){

							settings.isMouseDown[key] = true;
							var positionX = e.offsetX;

							if( e.target != this ){
						        positionX = e.target.offsetLeft + e.offsetX;
						    }

							var array_index = updateThumbPosition(positionX, key);
							hideThumbPosition(this, array_index);
							settings.results[key] = ranges[array_index];
							localStorage.setItem('rangerResults', JSON.stringify(settings.results));
							
							return false;
						};

						parents[pindex].onmousemove = function(e){

							if(settings.isMouseDown[key]){

								var positionX = e.offsetX;

								if( e.target != this ){
							        positionX = e.target.offsetLeft + e.offsetX;
							    }

								var array_index = updateThumbPosition(positionX, key);
								hideThumbPosition(this, array_index);
								settings.results[key] = ranges[array_index];
								localStorage.setItem('rangerResults', JSON.stringify(settings.results));
							}
						}

						parents[pindex].onmouseup = function(e){
							settings.isMouseDown[key] = false;
						}

						window.onmouseup = function(){

							for (var key in settings.isMouseDown) {
							    if (settings.isMouseDown.hasOwnProperty(key)) {
							        settings.isMouseDown[key] = false;
							    }
							}
						}

					})(i);
				}		
			}
		}
		// End of get Elements
		
		function getSmallestDistance(offset, array)
		{
			return "((100% - " + offset + "px) / " + (array.length - 1) + ")";
		}
		// End of getSmallestDistance
		
		function updateThumbPosition(offset, key)
		{
        	var sliderPosition = offset;
			var closest = getClosestNumber(sliderPosition, settings.markerPosition[key]);
			return settings.markerPosition[key].indexOf(closest);
        }
        // End of updateThumbPosition
		
		function getClosestNumber(num, arr)
		{
            var curr = arr[0];
            var diff = Math.abs (num - curr);
            for (var val = 0; val < arr.length; val++) {
                var newdiff = Math.abs (num - arr[val]);
                if (newdiff < diff) {
                    diff = newdiff;
                    curr = arr[val];
                }
            }
            return curr;
        }
        // End of getClosestNumber
        
        function hideThumbPosition(parent, index)
        {
        	var thumbs = parent.getElementsByClassName(settings.class);
        	for (var i = 0; i < thumbs.length; i++) {

        		if (i != index){
        			thumbs[i].classList.remove('tactive');
        		} else {
        			thumbs[i].classList.add('tactive');
        		}
        	}
        }
        // End of hideThumbPosition
		
		library.getSelected = function(){
			return localStorage.getItem('rangerResults');
		}

		return library;
	}

	if(typeof(window.Ranger === 'undefined')){
		window.Ranger = ranger();
	}

})(window);